import time
import rich
from hurry.filesize import size
from tabulate import tabulate


class Format:
    def __init__(self):
        pass

    @staticmethod
    def cli(**kwargs):
        """
        Handle in coming CLI Output Style
        """
        if "standard" in kwargs["style"]:
            return Format._default(**kwargs)
        return "Error"

    @classmethod
    def _default(cls, **kwargs):
        table = []
        columns = []
        rows = []
        if kwargs["source"] == "dict":
            columns, rows = cls.__dict(kwargs["data"])
            table.append(rows)

        elif kwargs["source"] == "list":
            for d in kwargs["data"]["response"]:
                columns, rows = cls.__list(d)
                table.append(rows)

        elif kwargs["source"] == "history":
            for d in kwargs["data"]["response"]:
                __columns, __rows = cls.__history(d)
                
                """ Don't override existing table """
                if len(__columns) != 0:
                    columns = __columns
                if len(__rows) != 0:
                    rows = __rows
                
                table.append(rows)

        elif kwargs["source"] == "progress":
            for d in kwargs["data"]["response"]:
                columns, rows = cls.__progress(d)
                table.append(rows)

        if len(rows) != 0:
            console = rich.get_console()
            console.print(
                tabulate(
                    table,
                    columns,
                    tablefmt="plain",
                    stralign="left",
                    disable_numparse=True,
                ),
                soft_wrap=True,
            )
            return True
        return False

    @classmethod
    def __dict(cls, data):
        columns = []
        rows = []
        for k, v in data.items():
            if k.lower() != "token":
                columns.append(k.upper())
                rows.append(v)
        return columns, rows

    @classmethod
    def __list(cls, d):
        excluded_columns = [
            "versions",
            "backup_services",
            "compatible_error",
        ]
        columns = []
        rows = []
        for k, v in d.items():
            if k not in excluded_columns:
                columns.append(k.upper())
                if "time" in k:
                    v = str(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(v)))
                if "size" in k:
                    v = str(size(v))
                if "percentage" in k:
                    v = "{}%".format(int(v))
                if "_version" in k:
                    v = str(v)
                rows.append(v)
        return columns, rows

    @classmethod
    def __history(cls, d):
        included_columns = [
            "backup_id",
            "start_timestamp",
            "status",
            "operation",
            "id",
            "progress_in_percentage",
            "description",
            "backup_size",
        ]
        columns = []
        rows = []
        if "PENDING" not in d["status"]:
            for k, v in d.items():
                if k in included_columns:
                    columns.append(k.upper())
                    if "time" in k:
                        v = str(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(v)))
                    if "size" in k:
                        v = str(size(v))
                    if "percentage" in k:
                        v = "{}%".format(int(v))
                    rows.append(v)
        return columns, rows

    @classmethod
    def __progress(cls, d):
        included_columns = [
            "backup_id",
            "start_timestamp",
            "status",
            "operation",
            "id",
            "progress_in_percentage",
            "description",
            "backup_size",
        ]
        columns = []
        rows = []
        for k, v in d.items():
            if k in included_columns:
                columns.append(k.upper())
                if "time" in k:
                    v = str(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(v)))
                if "size" in k:
                    v = str(size(v))
                if "percentage" in k:
                    v = "{}%".format(int(v))
                rows.append(v)
        return columns, rows



import logging
import os
from datetime import timezone, timedelta, datetime
from collections import OrderedDict
import json
import requests
import schedule
from requests.auth import HTTPBasicAuth
import urllib3
from rich.console import Console
import click
from ciscodnacbackupctl.format import Format
from ciscodnacbackupctl.config import Config
from ciscodnacbackupctl.debug import Debug

author = "Robert Csapo"
email = "rcsapo@cisco.com"
description = "Cisco DNA Center Backup CLI"
repo_url = "https://github.com/cskoglun/ciscodnacbackupctl"
copyright = "Copyright (c) 2020 Cisco and/or its affiliates."
license = "Cisco Sample Code License, Version 1.1"
version = "0.2.11"


class Api:
    def __init__(self, config=False):
        """
        Create client session
        Checking config (env/local file)
        """
        _client = Config()
        if _client.config[0] is False:
            return
        """
        Authenticate towards Cisco DNA Center
        """
        if config is False:
            self.settings = _client.config[1]
            self._auth()
        return

    @classmethod
    def config(cls, hostname, username, password, secure, **kwargs):
        """
        if "read" in kwargs["operation"]:
            return Config.read()
        """
        if "write" in kwargs["operation"]:
            return Config.write(hostname, username, password, secure, **kwargs)

        return

    def _auth(self):
        """Cisco DNA Center Auth"""
        url = "https://{}/dna/system/api/v1/auth/token".format(
            self.settings["dnac"]["hostname"]
        )
        logging.info("Cisco DNA Center Authentication ({})".format(url))
        data = self._request(type="auth", url=url)
        self.settings["dnac"]["token"] = data["Token"]
        return

    def get(self, reverse=False):
        """
        Get Cisco DNA Center Backups
        """
        url = "https://{}{}".format(
            self.settings["dnac"]["hostname"], "/api/system/v1/maglev/backup"
        )

        data = self._request(type="get", url=url)
        data["response"] = sorted(
            data["response"], key=lambda k: k["end_timestamp"], reverse=reverse
        )

        data = self.remove_columns(data)

        return data

    def remove_columns(self, data):

        i = 0
        remove_columns = ["tenantId", "_id", "_version"]
        for backup in data["response"]:
            for k in list(backup):
                if k in remove_columns:
                    del data["response"][i][k]
            data["response"][i] = OrderedDict(sorted(backup.items()))
            i += 1
        return data

    def get_history(self):
        url = "https://{}{}".format(
            self.settings["dnac"]["hostname"], "/api/system/v1/maglev/backup/history"
        )
        data = self._request(type="get", url=url)
        data = self.remove_columns(data)
        return data

    def get_progress(self):
        url = "https://{}{}".format(
            self.settings["dnac"]["hostname"], "/api/system/v1/maglev/backup/progress"
        )
        data = self._request(type="get", url=url)
        data = self.remove_columns(data)
        return data

    def schedule_interval(self, **kwargs):
        """
        Function that returns the correct Schedule function based on which interval day that has been chosen
        """

        intervals = ["weekly", "daily"]

        if kwargs["interval"].lower() == intervals[0]:
            if kwargs["day"].lower() == "monday":
                ret = schedule.every().monday.at(kwargs["time"])
            if kwargs["day"].lower() == "tuesday":
                ret = schedule.every().tuesday.at(kwargs["time"])
            if kwargs["day"].lower() == "wednesday":
                ret = schedule.every().wednesday.at(kwargs["time"])
            if kwargs["day"].lower() == "thursday":
                ret = schedule.every().thursday.at(kwargs["time"])
            if kwargs["day"].lower() == "friday":
                ret = schedule.every().friday.at(kwargs["time"])
            if kwargs["day"].lower() == "saturday":
                ret = schedule.every().saturday.at(kwargs["time"])
            if kwargs["day"].lower() == "sunday":
                ret = schedule.every().sunday.at(kwargs["time"])
        elif kwargs["interval"].lower() == intervals[1]:
            ret = schedule.every().day.at(kwargs["time"])
        else:
            raise Exception

        return ret

    def _request(self, **kwargs):
        urllib3.disable_warnings()
        """
        HTTP Requests
        """

        if "auth" in kwargs["type"].lower():
            url = kwargs["url"]
            headers = {
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                auth=HTTPBasicAuth(
                    self.settings["dnac"]["username"],
                    self.settings["dnac"]["password"],
                ),
                headers=headers,
                verify=self.settings["dnac"]["secure"],
            )

            Debug.payload(str(response.text))

            if response.ok:
                data = response.json()
            else:
                if response.status_code == 429:
                    raise Exception(
                        "Can't login to Cisco DNA Center - Too Many Requests - Please try later"
                    )
                raise Exception(
                    "Can't login to Cisco DNA Center ({})".format(response.json())
                )
            return data

        if "get" in kwargs["type"].lower():
            """
            HTTP Method GET
            """
            url = kwargs["url"]
            headers = {
                "X-Auth-Token": self.settings["dnac"]["token"],
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
            response = requests.request(
                "GET",
                url,
                headers=headers,
                verify=self.settings["dnac"]["secure"],
            )

            Debug.payload(str(response.text))

            response.raise_for_status()
            if response.ok:
                data = response.json()
            else:
                if response.status_code == 404:
                    raise Exception("Error: Not found")
                data = response.json()
                print(
                    "Error: ({})".format(data["response"].get("error", "Not Available"))
                )
            return data
        if "delete" in kwargs["type"].lower():
            """
            HTTP Method DELETE
            """
            url = kwargs["url"]
            headers = {
                "X-Auth-Token": self.settings["dnac"]["token"],
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
            response = requests.request(
                "DELETE",
                url,
                headers=headers,
                verify=self.settings["dnac"]["secure"],
            )

            Debug.payload(str(response.text))

            if response.ok:
                data = response.json()
            else:
                if response.status_code == 404:
                    if "response" in response.json():
                        raise Exception(
                            f"Error: Not found ({response.json()['response']})"
                        )
                    raise Exception(f"Error: Not found ({response.text})")
                data = response.json()
                raise Exception(
                    "Error: ({})".format(data["response"].get("error", "Not Available"))
                )
            return data

        if "post" in kwargs["type"].lower():
            url = kwargs["url"]
            headers = {
                "X-Auth-Token": self.settings["dnac"]["token"],
                "Content-Type": "application/json",
                "Accept": "application/json",
            }
            response = requests.request(
                "POST",
                url,
                headers=headers,
                verify=self.settings["dnac"]["secure"],
                data=kwargs["payload"],
            )
            

            Debug.payload(str(response.text))

            if response.ok:
                data = response.json()
            elif response.status_code == 409:
                data = response.json()
            else:
                if response.status_code == 404:
                    if "response" in response.json():
                        raise Exception(
                            f"Error: Not found ({response.json()['response']})"
                        )
                    raise Exception(f"Error: Not found ({response.text})")
                data = response.json()
                raise Exception(
                    "Error: ({})".format(data["response"].get("error", "Not Available"))
                )
            return data

    class CLI:
        def __init__(self, debug=False):
            if debug:
                Debug()
            self.api = Api()
            

        def whoami(self, **kwargs):
            if "DNAC_CONFIG" in os.environ:
                self.api.settings["dnac"]["method"] = "environment (encoded)"
            elif "DNA_CENTER_BASE_URL" in os.environ:
                self.api.settings["dnac"]["method"] = "environment"
            else:
                self.api.settings["dnac"]["method"] = "file"
            Format.cli(
                style="standard",
                data=self.api.settings["dnac"],
                source="dict",
            )
            return True

        def list(self, reverse):
            data = self.api.get(reverse=reverse)
            Format.cli(style="standard", data=data, source="list")
            return True

        def history(self):
            data = self.api.get_history()
            Format.cli(style="standard", data=data, source="history")
            return True

        def create(self, name):
            console = Console()
            url = "https://{}{}".format(
                self.api.settings["dnac"]["hostname"], "/api/system/v1/maglev/backup"
            )
            payload = {"description": name}
            payload = json.dumps(payload)

            data = self.api._request(type="post", url=url, payload=payload)

            if type(data["response"]) == str:
                message = data["response"]
                console.log(f"Creating backup with id: {message}")
            else:
                message = data["response"]["error"]
                console.log(f"Error: {message}")

            return True

        def progress(self):
            """
            Cisco DNA Center Backups in Progress
            """
            data = self.api.get_progress()
            Format.cli(style="standard", data=data, source="progress")
            return True

        def delete(self, backup_id):
            console = Console()
            """
            Task to delete backups, either from str or list
            """
            def task(id):
                if len(id) != 36:
                    raise ValueError(f"Invalid Backup ID - {id}")
                url = "https://{}{}{}".format(
                    self.api.settings["dnac"]["hostname"],
                    "/api/system/v1/maglev/backup/",
                    id,
                )
                data = self.api._request(type="delete", url=url)

                if "status" in data["response"]:
                    if data["response"]["status"] == "ok":
                        console.log(data["response"]["message"])
                else:
                    console.log("Error: {}".format(data["response"]))

                return True
            if isinstance(backup_id, str):
                task(backup_id)
            elif isinstance(backup_id, list):
                for id in backup_id:
                    task(id)
            elif isinstance(backup_id, tuple):
                for id in backup_id:
                    task(id)
            else:
                raise TypeError(f"Incorrect type for backup_id - {type(backup_id)}")

            return True

        def schedule_backup(self, **kwargs):
            name = kwargs["name"]
            url = "https://{}{}".format(
                self.api.settings["dnac"]["hostname"],
                "/api/system/v1/maglev/schedule/backup",
            )
            url_post = url + f"/{name}"
            existing_data = self.api._request(type="get", url=url)

            if not existing_data["response"]:

                if kwargs["action"] == "create":
                    """
                    Create list of number 0-6 representing each weekday. This is needed for the DNAC payload.
                    """
                    weekdays = [
                        "sunday",
                        "monday",
                        "tuesday",
                        "wednesday",
                        "thursday",
                        "friday",
                        "saturday",
                    ]

                    day_list = kwargs["day"]
                    if len(day_list) >= 1:
                        if day_list[0].lower() == "everyday":
                            _list = []
                            for day in weekdays:
                                _list.append(str(weekdays.index(day)))
                            interval = ",".join(_list)

                        else:
                            days = list(day_list)
                            _list = []

                            for day in days:
                                day = day.lower()
                                _list.append(str(weekdays.index(day)))
                            interval = ",".join(_list)
                    else:
                        raise Exception(
                            "Error in day argument when using schedule_backup()"
                        )

                    time = kwargs["time"]
                    time_list = time.split(":")
                    time_list[0] = int(time_list[0]) - 1
                    """
                    Payload for the POST request
                    """
                    payload = {
                        "schedule": "{} {} * * {}".format(
                            time_list[1], time_list[0], interval
                        ),
                        "json_payload": {
                            "description": kwargs["name"],
                            "appstacks": {"ndp": {}},
                        },
                        "env": {},
                        "url": "http://glusterfs-brick.maglev-system.svc.cluster.local:8080/api/v1/sidecar/backup/1234",
                    }
                    payload = json.dumps(payload)

                    data = self.api._request(type="post", url=url_post, payload=payload)
                    message = "There is now a scheduled backup"
                    data = [message, name, None]
                else:
                    message = "There is no scheduled backup available to delete"
                    data = [message]

            else:
                if kwargs["action"] == "create":
                    message = "There already exists a scheduled backup"
                    name = existing_data["response"][0]["name"]
                    ts = int(existing_data["response"][0]["upcoming_run"])
                    upcoming_time = datetime.utcfromtimestamp(ts).strftime(
                        "%Y-%m-%d %H:%M"
                    )
                    data = [message, name, upcoming_time]
                else:
                    if existing_data["response"][0]["name"] == name:
                        data = self.api._request(type="delete", url=url_post)
                        message = f"Backup with the name '{name}' has been deleted"
                        data = [message]
                    else:
                        message = f"No backup with the name '{name}' exists."
                        data = [message]

            return data

        def backup_delta(self, data, days):
            """ Calc delta times to keep backups based on days """
            timenow = datetime.now(tz=timezone.utc)
            keep = days[:-1]
            delta = timenow - timedelta(int(keep))
            delta_ts = datetime.timestamp(delta)
            res = [b for b in data["response"] if b["end_timestamp"] >= delta_ts]
            return res

        def backups_to_delete(self, **kwargs):
            """
            Purge (delete) Cisco DNA Center Backups
            """

            data = self.api.get(reverse=True)

            """
            List of {backup_ids} to keep
            """
            backups_to_keep = []

            if kwargs["incompatible"]:
                """
                Amount of {backup_ids} that is compatible
                """

                for backup in data["response"]:
                    if backup["compatible"].upper() == "TRUE":
                        backups_to_keep.append(backup)
            elif "d" in str(kwargs["keep"]):
                """
                Amount of {backup_ids} to keep based on days
                """
                backups = self.backup_delta(data, kwargs["keep"])
                for backup in backups:
                    backups_to_keep.append(backup)
            else:
                """
                Amount of {backup_ids} to keep
                """
                i = 0
                for backup in data["response"]:
                    if i != int(kwargs["keep"]):
                        backups_to_keep.append(backup)
                        i += 1
                    else:
                        break
            """
            Removing {backup_ids} in {backups_to_keep} from data["response"]
            """
            for backup in backups_to_keep:
                data["response"].remove(backup)

            return data

        def purge(self, keep, incompatible, force):
            """
            Purge (delete) Cisco DNA Center Backups
            """
            console = Console()
            data = self.backups_to_delete(
                incompatible=incompatible, keep=keep, force=force
            )  # backups_to_delete

            if len(data["response"]) == 0:
                console.log("No backup to delete")
                return False
            else:
                backup_id_to_delete = []
                for item in range(0, len(data["response"])):
                    backup_id_to_delete.append(data["response"][item]["backup_id"])

            if force is True:
                """
                Displayed deleted backups
                """
                for i in range(0, len(backup_id_to_delete)):
                    self.delete(backup_id_to_delete[i])
                Format.cli(style="standard", data=data, source="list")
                console.log(
                    f"Success: Backups ({len(data['response'])}) deleted", style="green"
                )
                return True
            else:
                """
                Display candidates to be deleted
                """
                Format.cli(style="standard", data=data, source="list")

                """
                Confirm action to delete
                """
                confirm = click.prompt(
                    click.style(
                        "Warning: Confirm if you want to delete these backups (y/n)",
                        fg="red",
                    ),
                    type=bool,
                )
                if not confirm:
                    console.log("Warning: Purge aborted", style="red")
                    return True

                """
                Purging backups with force
                """
                console.log(
                    "Deleting... (this could take a while - as it's synchronous API calls)",
                    style="red",
                )

                data = self.backups_to_delete(
                    incompatible=incompatible, keep=keep, force=True
                )

                backup_id = [x["backup_id"] for x in data["response"]]
                self.delete(backup_id)
                console.log(
                    f"Success: Backups ({len(data['response'])}) deleted {backup_id}",
                    style="green",
                )
                return True



import logging
import sys
from http.client import HTTPConnection


class Debug:
    """Debug to Console"""

    def __init__(self) -> None:
        """Setup"""
        self.filename = "debug.txt"
        HTTPConnection.debuglevel = 2
        logging.basicConfig(
            level=logging.DEBUG,
            format="%(asctime)s %(levelname)s: %(message)s",
            stream=sys.stdout,
        )
        requests_log = logging.getLogger("requests.packages.urllib3")
        requests_log.setLevel(logging.DEBUG)
        requests_log.propagate = True
        pass

    @staticmethod
    def payload(data):
        """HTTP Payload"""
        logger = logging.getLogger(__name__)
        logger.debug(str(data))



import time
import click
import ciscodnacbackupctl
import rich
from rich.console import Console
import daemonocle
import schedule
from ciscodnacbackupctl import Format
from ciscodnacbackupctl import Config

# Need to pass the KEEP variable to daemon worker
KEEP = 3


def app_daemon(
    interval="daily", incompatible=False, keep=KEEP, day="monday", hour="23:00"
):
    global KEEP
    ctl = ciscodnacbackupctl.Api()
    interval = interval.lower()
    _day = day.lower()
    _time = hour.lower()
    """
    job() function makes sure to purge the backups when called upon
    """

    def job():
        console = Console()
        ctl = ciscodnacbackupctl.Api()
        cli = ctl.CLI()
        force = True
        try:
            purge = cli.purge(keep=KEEP, incompatible=incompatible, force=force)
        except Exception as error_msg:
            console.log(
                "Cisco DNA Center isn't available - {}".format(error_msg), style="green"
            )
            pass

    schedule_function = ctl.schedule_interval(
        interval=interval, day=_day, time=_time
    ).do(job)
    while True:
        schedule.run_pending()
        time.sleep(1)
    return

@click.group()
@click.option('--debug/--no-debug', default=False)
@click.version_option()
@click.pass_context
def cli(ctx, debug):
    ctx.obj["DEBUG"] = debug
    if debug:
        print("Debug mode: {}".format(debug))
    pass

@cli.command("daemon", help="{}".format(daemonocle.Daemon.list_actions()))
@click.option("--detach/--debug", default=True, help="Attach and debug")
@click.option(
    "--keep", type=str, default=3, help="Amount of backups to keep (default 3)"
)
@click.argument("command", nargs=1)
@click.pass_context
def daemon(ctx, detach, keep, command):
    global KEEP
    KEEP = keep
    daemon = daemonocle.Daemon(
        worker=app_daemon,
        pid_file="/tmp/ciscodnacbackupctl.pid",
        detach=detach,
    )
    if command.lower() in daemonocle.Daemon.list_actions():
        if command.lower() == "status":
            data = daemon.get_status()
            output = Format.cli(style="standard", data=data, source="dict")
            return
        daemon.do_action(command)
        return
    click.echo(ctx.get_help())
    return


@cli.command("whoami")
@click.option("--cfg", is_flag=True, help="Read local cfg")
@click.pass_context
def whoami(ctx, cfg):
    cli = ciscodnacbackupctl.Api.CLI(debug=ctx.obj["DEBUG"])
    cli.whoami(cfg=cfg)
    return


@cli.command("config")
@click.option("--env/--no-env", default=False)
@click.option("--hostname", required=True)
@click.option("--username", required=True)
@click.option("--password", required=True)
@click.option("--secure", is_flag=True, help="Secure HTTPS towards Cisco DNA Center")
@click.option(
    "--encode", is_flag=True, help="Encode Cisco DNA Center config to Base64 string"
)
@click.option(
    "--overwrite",
    is_flag=True,
    help="Writes over the existing config for Cisco DNA Center",
)
@click.pass_context
def config(ctx, env, hostname, username, password, secure, overwrite, encode):
    ctl = ciscodnacbackupctl.Api(config=True)
    console = Console()
    if env:
        if encode:
            data = {
                "dnac": {
                    "hostname": hostname,
                    "username": username,
                    "password": password,
                    "secure": secure,
                }
            }
            data = Config.base64_cfg(operation="encode", data=data)
            environments = {
                "DNAC_CONFIG": data[1],
            }
        else:
            environments = {
                "DNA_CENTER_BASE_URL": hostname,
                "DNA_CENTER_USERNAME": username,
                "DNA_CENTER_PASSWORD": password,
                "DNA_CENTER_VERIFY": secure,
            }
        console.print(
            "Environment Settings Generated (copy paste below)", style="green"
        )
        for k, v in environments.items():
            console.print("export {}={}".format(k, v), soft_wrap=True)
    else:
        """ TODO move to config.py """
        config = ciscodnacbackupctl.Api.config(
            hostname=hostname,
            username=username,
            password=password,
            secure=secure,
            operation="write",
        )

        if "Config already exist" in str(config[1]) and config[0] is False:
            if overwrite is False:
                confirm = click.prompt(
                    click.style(
                        "Warning: Config already exist ({}), are you sure you want override? (y/n)".format(
                            ciscodnacbackupctl.config.DNAC_FULL_CONFIG_PATH
                        ),
                        fg="red",
                    ),
                    type=bool,
                )
            if not confirm:
                console.print("Warning: Config aborted", style="red")
                return
            else:
                config = ciscodnacbackupctl.Api.config(
                    hostname=hostname,
                    username=username,
                    password=password,
                    secure=secure,
                    operation="overwrite",
                )
        console.print(
            "Success: Config created ({})".format(
                ciscodnacbackupctl.config.DNAC_FULL_CONFIG_PATH
            ),
            style="green",
        )
    return


@cli.command("list")
@click.option("--reverse/--no-reverse", default=False)
@click.pass_context
def list(ctx, reverse):
    cli = ciscodnacbackupctl.Api.CLI(debug=ctx.obj["DEBUG"])
    cli.list(reverse)
    return


@cli.command("progress")
@click.pass_context
def progress(ctx):
    cli = ciscodnacbackupctl.Api.CLI(debug=ctx.obj["DEBUG"])
    cli.progress()
    return


@cli.command("history")
@click.pass_context
def history(ctx):
    cli = ciscodnacbackupctl.Api.CLI(debug=ctx.obj["DEBUG"])
    cli.history()
    return


@cli.command("create")
@click.option("--name", required=True, help="Name of backup")
@click.pass_context
def create(ctx, name):
    cli = ciscodnacbackupctl.Api.CLI(debug=ctx.obj["DEBUG"])
    cli.create(name)
    return


@cli.command("delete")
@click.option("--backup_id", required=True, is_flag=True)
@click.argument('id', nargs=-1)
#@click.argument("--backup_id", required=True, nargs=-1)
@click.pass_context
def delete(ctx, backup_id, id):
    cli = ciscodnacbackupctl.Api.CLI(debug=ctx.obj["DEBUG"])
    cli.delete(id)
    return


@cli.command("schedule_backup")
@click.option(
    "--action",
    "-a",
    required=True,
    help="Type "
    "Create"
    " or "
    "Delete"
    " to either create or delete scheduled backup",
    type=str,
)
@click.option("--name", "-n", required=True, help="Name of backup", type=str)
@click.option(
    "--day",
    "-d",
    multiple=True,
    help="Type which day you want backup or "
    "everyday"
    " if you want everyday backup.",
    default=["everyday"],
)
@click.option(
    "--hour",
    "-t",
    required=False,
    default="23:00",
    type=str,
    help="At what time hh:mm should the backup be created? Default is 23:00",
)
@click.pass_context
def schedule_backup(ctx, action, name, day, hour):
    action = action.lower()
    cli = ciscodnacbackupctl.Api().CLI(debug=ctx.obj["DEBUG"])
    console = rich.get_console()
    data = cli.schedule_backup(name=name, day=day, time=hour, action=action)
    print(data)

    # TODO move this part to CLI class
    if action.lower() == "create":
        if "now" in data[0]:
            console.print(f"{data[0]} with name '{data[1]}'", style="green")
        else:
            console.print(f"{data[0]} with name '{data[1]}' ", style="red")
    else:
        if "no" in data[0]:
            console.print(f"{data[0]}", style="red")
        else:
            console.print(f"{data[0]}", style="red")


@cli.command("purge")
@click.option(
    "--keep", type=str, default=3, help="Amount of backups to keep (default 3)"
)
@click.option("--incompatible", is_flag=True, help="Remove incompatible backups")
@click.option("--force", is_flag=True, help="No interactive prompt to confirm purge")
@click.pass_context
def purge(ctx, keep, incompatible, force):
    cli = ciscodnacbackupctl.Api().CLI(debug=ctx.obj["DEBUG"])
    purge = cli.purge(keep=keep, incompatible=incompatible, force=force)


@cli.command("schedule_purge")
@click.option(
    "--interval", "-i", required=True, help=" " "daily" " or " "weekly" " ", type=str
)
@click.option("--incompatible", is_flag=True, help="Remove incompatible backups")
@click.option(
    "--keep",
    "-k",
    required=False,
    default=3,
    type=str,
    help="Amount of backups to keep, default is 3",
)
@click.option(
    "--day",
    "-d",
    required=False,
    default="monday",
    type=str,
    help="Day to delete backups on",
)
@click.option(
    "--hour",
    "-h",
    required=False,
    default="23:00",
    type=str,
    help="Time to delete backups HH:MM, default is 23:00",
)
@click.pass_context
def schedule_purge(ctx, interval, incompatible, keep, day, hour):
    ctl = ciscodnacbackupctl.Api()
    cli = ctl.CLI(debug=ctx.obj["DEBUG"])
    console = rich.get_console()

    interval = interval.lower()
    _day = day.lower()
    _time = hour.lower()

    """
    job() function makes sure to purge the backups when called upon
    """

    def job():
        force = True
        try:
            purge = cli.purge(keep=keep, incompatible=incompatible, force=force)
        except Exception as error_msg:
            console.log(
                "Cisco DNA Center isn't available - {}".format(error_msg), style="green"
            )
            pass

    """
    Print out confirmation that the backup is scheduled
    """
    # TODO move this part to class CLI
    if interval.lower() == "daily":
        click.echo(
            click.style(f"\nYour backups will be deleted daily at {_time}", fg="green")
        )
    else:
        click.echo(
            click.style(
                f"\nYour backups will be deleted {_day}s at {_time}", fg="green"
            )
        )

    schedule_function = ctl.schedule_interval(
        interval=interval, day=_day, time=_time
    ).do(job)

    while True:
        schedule.run_pending()
        time.sleep(1)


if __name__ == "__main__":
    cli(obj={})

def entry():
    cli(obj={})


from pathlib import Path
import os
import json
import base64

DNAC_CONFIG = "config.json"
DNAC_CONFIG_PATH = "/.ciscodnac/"
DNAC_FULL_CONFIG_PATH = "{}{}{}".format(Path.home(), DNAC_CONFIG_PATH, DNAC_CONFIG)


class Config:
    def __init__(self):
        self.config = self._check_settings()
        pass

    def _check_settings(self):
        """
        Check Environment cfg
        """
        env_cfg = self.read_env()
        if env_cfg[0] is True:
            return env_cfg[0], env_cfg[1]
        """
        Check local file cfg
        """
        file_cfg = self.read_file()
        if file_cfg[0] is True:
            return file_cfg[0], file_cfg[1]

        return False, "Can't find Cisco DNA Center Config"

    @classmethod
    def read_env(cls):
        """
        Mandatory Environment variables if using Base64
        """
        enviroment_cfg_base64 = "DNAC_CONFIG"
        if enviroment_cfg_base64 in os.environ:
            """ Validate cfg as base64 string """
            data = cls.base64_cfg(operation="decode", data=os.environ["DNAC_CONFIG"])
            if data[0] is True:
                return True, data[1]
            return False, data[1]
        """
        Mandatory Environment variables
        """
        enviroment_cfg = [
            "DNA_CENTER_BASE_URL",
            "DNA_CENTER_USERNAME",
            "DNA_CENTER_PASSWORD",
        ]
        valid_env_cfg = True
        for env in enviroment_cfg:
            if env not in os.environ:
                valid_env_cfg = False
                break

        """
        Validate cfg
        """
        if valid_env_cfg:
            if "DNA_CENTER_VERIFY" in os.environ:
                if "true" in os.environ["DNA_CENTER_VERIFY"].lower():
                    secure = True
                else:
                    secure = False
            else:
                secure = False
            data = {
                "dnac": {
                    "hostname": os.environ["DNA_CENTER_BASE_URL"],
                    "username": os.environ["DNA_CENTER_USERNAME"],
                    "password": os.environ["DNA_CENTER_PASSWORD"],
                    "secure": secure,
                }
            }
            return True, data

        return False, "Environment variables missing"

    @classmethod
    def base64_cfg(cls, **kwargs):
        """ Handle base64 encoding """
        if "encode" in kwargs["operation"]:
            """ Encode json cfg to base64 str """
            data = str(json.dumps(kwargs["data"], indent=4))
            data = base64.b64encode(data.encode("ascii"))
            return True, data.decode("utf-8")
        if "decode" in kwargs["operation"]:
            """ Decode base64 str to json cfg """
            data = kwargs["data"]
            try:
                data == base64.b64encode(base64.b64decode(data)).decode("utf-8")
                data = json.loads(base64.b64decode(data).decode("utf-8"))
            except Exception as e:
                return False, f"Can't decode config ({e})"
            return True, data
        return False, "Unknown base64 error"

    @classmethod
    def read_file(cls):
        """
        Reading local cfg file (json)
        """
        try:
            with open(DNAC_FULL_CONFIG_PATH, "r") as f:
                data = f.read()
                f.close()
                try:
                    """ Read base64 encoded config """
                    data == base64.b64encode(base64.b64decode(data)).decode("utf-8")
                    data = json.loads(base64.b64decode(data).decode("utf-8"))
                except Exception:
                    try:
                        """ Read JSON config (if not base64 encoded) """
                        data = json.loads(data)
                    except Exception:
                        return False, "Can't load json from config"
                return True, data
        except Exception as e:
            return False, f"Can't read config file ({e})"

    @classmethod
    def write(cls, hostname, username, password, secure, **kwargs):
        """
        Write config to local file (json)
        """

        """
        Check if existing folder structure exist
        """
        local_user_path = "{}{}".format(Path.home(), DNAC_CONFIG_PATH)
        if os.path.exists(local_user_path) is False:
            os.makedirs("{}{}".format(Path.home(), DNAC_CONFIG_PATH))

        """
        Don't override existing cfg without {overwrite}
        """
        if (
            "overwrite" not in kwargs["operation"]
            and cls._config_exist(DNAC_FULL_CONFIG_PATH) is True
        ):
            return False, "Config already exist"

        """
        Write json config to file
        """
        data = {
            "dnac": {
                "hostname": hostname,
                "username": username,
                "password": password,
                "secure": secure,
            }
        }
        try:
            with open(DNAC_FULL_CONFIG_PATH, "w") as f:
                f.write(json.dumps(data, indent=4))
            f.close()
            return True, "Success"
        except Exception as e:
            return False, f"Can't update config file ({e})"

    @classmethod
    def _config_exist(cls, path):
        """
        Check if local config file already exist
        """
        if os.path.exists(path):
            return True
        return False



from os import path
from setuptools import setup, find_packages
import ciscodnacbackupctl

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

with open(
    path.join(path.abspath(path.dirname(__file__)), "README.md"), encoding="utf-8"
) as f:
    long_description = f.read()

setup(
    name="ciscodnacbackupctl",
    author=ciscodnacbackupctl.author,
    author_email=ciscodnacbackupctl.email,
    description=ciscodnacbackupctl.description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=ciscodnacbackupctl.repo_url,
    version=ciscodnacbackupctl.version,
    packages=find_packages(),
    py_modules=["ciscodnacbackupctl"],
    install_requires=requirements,
    entry_points="""
        [console_scripts]
        ciscodnacbackupctl=ciscodnacbackupctl.cli:entry
    """,
    python_requires=">=3.8",
)
